import h5py
import numpy as np
import scipy.sparse as sparse

def h5loader(h5_path: str, chr_pair: str, resolution: int) -> sparse.csr_matrix:
	print(h5_path)
	with h5py.File(h5_path, 'r') as h5file:
		chr1,chr2 = chr_pair.split('-')
		chr1_size = np.int32(h5file['chr_sizes'][chr1][0]//resolution)
		chr2_size = np.int32(h5file['chr_sizes'][chr2][0]//resolution)
		row = np.array(h5file['data'][chr_pair]['row'][:], dtype=np.int32)//resolution
		col = np.array(h5file['data'][chr_pair]['col'][:], dtype=np.int32)//resolution
		mat = sparse.csr_matrix((np.ones_like(row, dtype=np.int32), (row, col)), shape=(chr1_size, chr2_size))
	try:
		mat.shape
		return mat
	except:
		raise ValueError(f'Matrix is empty because non of {h5_path} contains {chr_pair}')

def randomBreakpoints(sample: str, type: str, chr_pair: str, sparse_matrix: sparse.csr_matrix, resolution: int):
	
	chr1,chr2 = chr_pair.split('-')
	true_breaks = [
		('model_18', 'chr10', 1085000, 'chr4', 89040000, 'translocation'),
		('model_18', 'chr10', 1145000, 'chr4', 89040000, 'translocation'),
		('model_18', 'chr13', 28875000, 'chr18', 8635000, 'translocation'),
		('model_18', 'chr13', 29195000, 'chr18', 8635000, 'translocation'),
		('model_18', 'chr14', 21990000, 'chr1', 24450000, 'translocation'),
		('model_18', 'chr14', 22040000, 'chr1', 24450000, 'translocation'),
		('model_18', 'chr16', 71165000, 'chr16', 71285000, 'copy_gain'),
		('model_18', 'chr2', 115215000, 'chr2', 130215000, 'copy_loss'),
		('model_18', 'chr8', 17805000, 'chr5', 137460000, 'translocation'),
		('model_18', 'chr8', 18345000, 'chr5', 137460000, 'translocation'),
		('model_18', 'chr8', 124330000, 'chr19', 53585000, 'translocation'),
		('model_18', 'chr8', 124370000, 'chr19', 53585000, 'translocation')
	]
	
	rearrangements = ['translocation','inversion','copy_gain','copy_loss']
	
	br1 = np.random.choice(sparse.find(sparse_matrix)[0],size=5)*resolution
	br2 = np.random.choice(sparse.find(sparse_matrix)[1],size=5)*resolution
	r_type = np.random.choice(rearrangements,size=5,replace=True)
	list_of_breaks = [ (sample, chr1, br1[i], chr2, br2[i], r_type[i]) for i in range(5)]
	list_of_breaks += [true_breaks[i] for i in np.random.choice(12,size=5,replace=False)]
	np.savetxt('results.tsv', list_of_breaks, delimiter='\t', fmt='%s')

	return

if __name__ == "__main__":
	
	import argparse
	parser = argparse.ArgumentParser(description='Parameters for generation of rearrangement maps')
	parser.add_argument( 'fname', metavar = 'file', help='the path to the metadata file')
	args = parser.parse_args()
	
	with open(args.fname, 'r') as f: lines = f.readlines()
	for line in lines[1:]: 
		sample,type,path,sex = line.split()
		chr_pair = 'chr13-chr14'
		resolution = 5000
		path = args.fname[:-10]+path[3:-1]
		matrix = h5loader(path, chr_pair, resolution)
		randomBreakpoints(sample, type,chr_pair,matrix,resolution)